/*
********************************************************************
(c) MSU Video Group 2003-2006, http://compression.ru/video/
This source code is property of MSU Graphics and Media Lab

This code may be distributed under LGPL
(see http://www.gnu.org/licenses/lgpl.html for more details).

E-mail: video-measure@compression.ru
Author: Alexey Noskov
********************************************************************
*/  

/*
* bi_psnr.cpp: implementation of VMT plugin for measurement of brigyhness-independent PSNR.
*/

#include "stdafx.h"

#include "pluginadapter.h"
#include "dfm.h"

template < typename T > T sqr ( const T & a ) { return a * a; }

/*
* DllMain
*/

BOOL APIENTRY DllMain ( HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved )
{
    return TRUE;
}

extern "C" __declspec(dllexport) int GetVQMTVersion()
{
	return 320;
}

extern "C" __declspec(dllexport) int CompatibleWithVQMT(int vqmtVer)
{
	return vqmtVer >= 320 ? 0 : -1;
}

extern "C" __declspec(dllexport) void CreateMetric ( IMetricPlugin**metric )
{
	*metric = new CPluginAdapter ( new CDFMPlugin );
}

extern "C" __declspec(dllexport) void ReleaseMetric ( IMetricPlugin* metric )
{
	delete metric;
}

/*
* Construction/destruction
*/

CDFMPlugin::CDFMPlugin ()
{
	m_plane_prev = NULL;
}

CDFMPlugin::~CDFMPlugin ()
{
	if ( m_plane_prev ) delete []m_plane_prev;
}

/*
* Initialization
*/

void CDFMPlugin::Init ( IMetricPlugin::COLORCOMPONENT cc, int width, int height, int &global_id )
{
	m_dfm = global_id++;

	m_width = width;
	m_height = height;
	m_size = width * height;

	m_frames_num = 0;
	m_frames_dropped = 0;

	m_plane_prev = new float [ m_size ];

	float * plane_prev = m_plane_prev;

	for ( int i = 0; i < m_size; ++ i, ++ plane_prev ) {
		//*plane_prev = 128.0f;
	}
}

/*
* Measurement
*/

vector< pair <IMetricPlugin::ID, float> > CDFMPlugin::Measure ( vector<IMetricImage*> &images )
{
	float * plane_prev = m_plane_prev;
	float * plane_cur = images [ 0 ]->GetY ();

	int is_dropped = 1;

	for ( int i = 0; i < m_size; ++ i, ++ plane_prev, ++ plane_cur ) {
		if ( abs ( *plane_prev - *plane_cur ) > 0.7f ) {
			is_dropped = 0;
			break;
		}
	}

	memcpy ( m_plane_prev, images [ 0 ]->GetY (), m_size * sizeof ( float ) );
	
	m_frames_num ++;
	m_frames_dropped += is_dropped;

	vector< pair <IMetricPlugin::ID, float> > res;

	res.push_back ( make_pair ( m_dfm, float ( is_dropped ) ) );
	return res;
}

vector< pair <IMetricPlugin::ID, float> > CDFMPlugin::MeasureAndVisualize ( vector<IMetricImage*>&images, BYTE *vis, int vis_pitch )
{
	float * plane_prev = m_plane_prev;
	float * plane_cur = images [ 0 ]->GetY ();

	int is_dropped = 1;

	for ( int i = 0; i < m_height; ++ i) 
    {
        for ( int j = 0; j < m_width; ++ j) 
	    {
		    if ( abs ( *plane_prev - *plane_cur ) > 0.7f ) is_dropped = 0;
		    BYTE val = BYTE ( *plane_cur - *plane_prev + 128 );
    		
		    SetRGB ( vis + 3 *j, 0, 0, val, val, val );
		    
            ++ plane_prev;
            ++ plane_cur;
	    }
	    vis += vis_pitch;
	}

	memcpy ( m_plane_prev, images [ 0 ]->GetY (), m_size * sizeof ( float ) );

	m_frames_num ++;
	m_frames_dropped += is_dropped;

	vector< pair <IMetricPlugin::ID, float> > res;

	res.push_back ( make_pair ( m_dfm, float ( is_dropped ) ) );
	return res;
}

vector< pair <IMetricPlugin::ID, float> > CDFMPlugin::CalculateAverage ( bool visualize )
{
	vector< pair <IMetricPlugin::ID, float> > res;
	
	res.push_back ( make_pair ( m_dfm, float ( m_frames_dropped ) ) );
	return res;
}

/*
* Private methods
*/

void CDFMPlugin::SetRGB ( BYTE * img, int x, int y, BYTE r, BYTE g, BYTE b )
{
	int ofs = ( y * m_width + x ) * 3;

	img [ ofs + 2 ] = r;
	img [ ofs + 1 ] = g;
	img [ ofs + 0 ] = b;
}

/*
* Config
*/

vector< pair <IMetricPlugin::ID, pair< vector <int>, wstring> > > CDFMPlugin::MapIDToFrame ( bool visualize )
{
	vector< pair <IMetricPlugin::ID, pair<vector <int>, wstring> > > res;

	/*
	* frames - vector of numbers of frames that given ID corresponds to. 
	* Here we have two IDs, both of which correspond to both frames 
	*/
	vector<int> frames;
	frames.push_back ( 0 );
	res.push_back ( make_pair ( m_dfm, make_pair ( frames, L"MSU DFM" ) ) );
	return res;
}

int CDFMPlugin::GetVideoNum ( bool visualize )
{
	return 1;
}

vector <IMetricPlugin::COLORCOMPONENT> CDFMPlugin::GetSupportedColorcomponents ()
{
	vector <IMetricPlugin::COLORCOMPONENT> cc;
	cc.push_back(IMetricPlugin::YYUV);
	return cc;
}

wstring CDFMPlugin::GetName ()
{
	return L"dfm";
}

wstring CDFMPlugin::GetUnit ()
{
	return L"";
}

int CDFMPlugin::GetWidthMultiply ()
{
	return 1;
}

int CDFMPlugin::GetHeightMultiply ()
{
	return 1;
}

wstring	CDFMPlugin::GetMetrInfoURL ()
{
	return L"http://compression.ru/video/quality_measure/metric_plugins/dfm_en.htm";
}

bool CDFMPlugin::GetMetrIncline ()
{
	return true;
}

wstring	CDFMPlugin::GetInterfaceName ()
{
	return L"MSU Drop Frame Metric v1.1";
}

bool CDFMPlugin::IsConfigurable ()
{
	return false;
}

bool CDFMPlugin::Config ( const wstring &configuration_string )
{
	return true;
}

void CDFMPlugin::GetDefaultConfiguration ( wstring &def_config )
{
	return;
}

//EOF//////////////////////////////////////////////////////////////////////////
